#include "utils.h"
#include "types.h"
#include "queue.h"
#include "stack.h"

#include <stdio.h>

void show(image img) {
	for (int i = 0; i < 50; i++) {
		for (int j = 0; j < 50; j++) {
			int color = img[i][j].color;

			if(color == 0) printf(" ");
			else if (color == 1) printf("@");
			else if (color == 2) printf("+");
			else printf("?");
		}

		printf("\n");
	}
}

pixel* voisin_gauche(pixel* p, image img) {
	if (p->i == 0) return NULL;

	return &img[p->i - 1][p->j];
}

pixel* voisin_droite(pixel* p, image img) {
	if (p->i == 49) return NULL;

	return &img[p->i + 1][p->j];
}

pixel* voisin_haut(pixel* p, image img) {
	if (p->i == 0) return NULL;

	return &img[p->i][p->j - 1];
}

pixel* voisin_bas(pixel* p, image img) {
	if (p->i == 49) return NULL;

	return &img[p->i][p->j + 1];
}

void floodfill_bfs(int i, int j, image img, int c) {
	if(img[i][j].color == c) return;

	queue* q = queue_create();

	queue_enqueue(q, &img[i][j]);
	int c0 = img[i][j].color;
	img[i][j].color = c;

	while (!queue_is_empty(q)) {
		pixel* p = queue_dequeue(q);

		pixel* g = voisin_gauche(p, img);
		pixel* d = voisin_droite(p, img);
		pixel* h = voisin_haut(p, img);
		pixel* b = voisin_bas(p, img);

		if(g != NULL && g->color != c && g->color == c0) { g->color = c; queue_enqueue(q, g); }
		if(d != NULL && d->color != c && d->color == c0) { d->color = c; queue_enqueue(q, d); }
		if(h != NULL && h->color != c && h->color == c0) { h->color = c; queue_enqueue(q, h); }
		if(b != NULL && b->color != c && b->color == c0) { b->color = c; queue_enqueue(q, b); }

		clear_screen();
		show(img);
		wait1();
	}

	queue_free(q);
}


void floodfill_dfs(int i, int j, image img, int c) {
	if(img[i][j].color == c) return;

	stack* s = stack_create();

	stack_push(s, &img[i][j]);

	int c0 = img[i][j].color;
	img[i][j].color = c;
	int k = 0;

	while (!stack_is_empty(s)) {
		pixel* p = stack_pop(s);

		pixel* g = voisin_gauche(p, img);
		pixel* d = voisin_droite(p, img);
		pixel* h = voisin_haut(p, img);
		pixel* b = voisin_bas(p, img);

		if(g != NULL && g->color != c && g->color == c0) { g->color = c; stack_push(s, g); }
		if(d != NULL && d->color != c && d->color == c0) { d->color = c; stack_push(s, d); }
		if(h != NULL && h->color != c && h->color == c0) { h->color = c; stack_push(s, h); }
		if(b != NULL && b->color != c && b->color == c0) { b->color = c; stack_push(s, b); }

		clear_screen();
		show(img);
		wait1();
	}

	stack_free(s);
}

int main() {
	image img = load_image("img1.txt");
	floodfill_bfs(30, 30, img, 2);
}
